import pandas as pd


def pivot_shipowners_associations(shipowner_shipment_with_name_df: pd.DataFrame) -> pd.DataFrame:
    df = pd.pivot_table(
        shipowner_shipment_with_name_df, index=["shipment_id"], columns=["role"], values=["name"], aggfunc="first"
    )
    df.columns = df.columns.droplevel()
    df = df.drop(["insurance", "other"], axis=1, errors="ignore")
    df = df.rename(
        columns={
            "arrival_fees": "arrival_shipowner",
            "arrival_truck_freight": "delivery_shipowner",
            "freight": "freight_shipowner",
            "departure_fees": "departure_shipowner",
            "departure_truck_freight": "pickup_shipowner",
        }
    )
    return df[
        ["pickup_shipowner", "departure_shipowner", "freight_shipowner", "arrival_shipowner", "delivery_shipowner"]
    ]


def add_shipowners_to_shipment(
    shipments_df: pd.DataFrame, shipowner_shipment_with_name_df: pd.DataFrame
) -> pd.DataFrame:
    shipowner_shipment_with_name_df_pivoted = pivot_shipowners_associations(shipowner_shipment_with_name_df)
    return pd.merge(
        shipments_df, shipowner_shipment_with_name_df_pivoted, left_on="shipment_id", right_index=True, how="left"
    )



import pandas as pd


def calculate_single_container_teus(single_container: pd.Series):
    if "twenty" in single_container["container_type"]:
        single_container["teus"] = 1
        return single_container
    single_container["teus"] = 2
    return single_container



import numpy as np
import pandas as pd
from ovretl.tracking_utils.extract_event_date import (
    extract_pickup_event_date,
    extract_arrival_event_date,
    add_tracking_event_dates,
)
from pandas.util.testing import assert_frame_equal


def test_extract_pickup_event():
    pickup_events_shipment_df = pd.DataFrame.from_records(
        [
            {
                "shipment_id": 0,
                "event_description": "Pickup",
                "date_type": "actual",
                "date": pd.Timestamp("2020-01-02 23:00:00"),
                "location_type": "warehouse",
            },
            {
                "shipment_id": 0,
                "event_description": "Pickup",
                "date_type": "estimated",
                "date": pd.Timestamp("2015-01-05 23:00:00"),
                "location_type": "warehouse",
            },
        ]
    )
    pickup_date = extract_pickup_event_date(events_shipment_df=pickup_events_shipment_df,)
    assert pickup_date == pd.Timestamp("2015-01-05 23:00:00")


def test_extract_arrival_event():
    arrival_events_shipment_df = pd.DataFrame.from_records(
        [
            {
                "shipment_id": 0,
                "event_description": "Arrival",
                "date_type": "actual",
                "date": pd.Timestamp("2020-01-02 23:00:00"),
                "is_used_for_eta": False,
                "location_type": "harbor",
            },
            {
                "shipment_id": 0,
                "event_description": "Arrival",
                "date_type": "estimated",
                "date": pd.Timestamp("2015-01-05 23:00:00"),
                "is_used_for_eta": True,
                "location_type": "harbor",
            },
        ]
    )
    arrival_date = extract_arrival_event_date(events_shipment_df=arrival_events_shipment_df,)
    assert arrival_date == pd.Timestamp("2015-01-05 23:00:00")


def test_add_tracking_events():
    events_shipment_df = pd.DataFrame.from_records(
        [
            {
                "shipment_id": 0,
                "event_description": "Pickup",
                "date_type": "actual",
                "date": pd.Timestamp("2020-01-02 23:00:00"),
                "location_type": "warehouse",
                "is_used_for_etd": False,
                "is_used_for_eta": False,
                "initial_eta": np.nan,
                "initial_etd": pd.Timestamp("2020-01-01 23:00:00"),
            },
            {
                "shipment_id": 0,
                "event_description": "Pickup",
                "date_type": "estimated",
                "date": pd.Timestamp("2015-01-05 23:00:00"),
                "location_type": "warehouse",
                "is_used_for_etd": False,
                "is_used_for_eta": False,
                "initial_eta": np.nan,
                "initial_etd": pd.Timestamp("2020-01-01 23:00:00"),
            },
        ]
    )
    shipments_df = pd.DataFrame(
        data={"shipment_id": [0], "cancelled": [False], "shipment_status": ["in_progress"], "finished_date": [np.nan],}
    )
    result = add_tracking_event_dates(shipments_df, events_shipment_df)
    result_should_be = pd.DataFrame(
        data={
            "shipment_id": [0],
            "cancelled": [False],
            "shipment_status": ["in_progress"],
            "finished_date": [np.nan],
            "pickup_date": [pd.Timestamp("2015-01-05 23:00:00")],
            "departure_date": [np.nan],
            "arrival_date": [np.nan],
            "delivery_date": [np.nan],
            "initial_eta": [np.nan],
            "initial_etd": [pd.Timestamp("2020-01-01 23:00:00")],
            "is_active": [True],
        }
    )
    assert_frame_equal(result, result_should_be)



import datetime

from ovretl.performances_utils.calculate_business_hours_delta import calculate_business_hours_delta


def test_calculate_business_hours_delta():
    # 2h + 3h
    date_1 = datetime.datetime(2020, 6, 26, 17, 45)
    date_2 = datetime.datetime(2020, 6, 29, 11, 30)
    assert calculate_business_hours_delta(date_1, date_2) == 4.75

    # 2h + 3h
    date_3 = datetime.datetime(2020, 6, 25, 17, 30)
    date_4 = datetime.datetime(2020, 6, 26, 11, 30)
    assert calculate_business_hours_delta(date_3, date_4) == 5

    # 11h + 6h
    date_5 = datetime.datetime(2020, 6, 25, 11, 30)
    date_6 = datetime.datetime(2020, 6, 26, 17, 30)
    assert calculate_business_hours_delta(date_5, date_6) == 17

    # 6h
    date_7 = datetime.datetime(2020, 6, 25, 11, 45)
    date_8 = datetime.datetime(2020, 6, 25, 17, 30)
    assert calculate_business_hours_delta(date_7, date_8) == 5.75

    # 11h + 2h + 3h
    date_9 = datetime.datetime(2020, 6, 25, 17, 30)
    date_10 = datetime.datetime(2020, 6, 27, 11, 30)
    assert calculate_business_hours_delta(date_9, date_10) == 16

    # 11h + 5h
    date_11 = datetime.datetime(2020, 6, 26, 17, 30)
    date_12 = datetime.datetime(2020, 6, 30, 11, 30)
    assert calculate_business_hours_delta(date_11, date_12) == 16



import pandas as pd


def calculate_margin_without_insurance(prices_df: pd.DataFrame):
    columns_to_exclude = ["insurance", "customs", "other"]
    prices_df_without_insurance = prices_df[~prices_df["category"].isin(columns_to_exclude)]
    prices_df_without_insurance.loc[:, "margin_without_insurance"] = prices_df_without_insurance.groupby("shipment_id")[
        "margin_price_in_eur"
    ].transform(sum)
    prices_df_without_insurance.loc[:, "initial_margin_without_insurance"] = prices_df_without_insurance.groupby(
        "shipment_id"
    )["initial_margin_price_in_eur"].transform(sum)
    prices_df_without_insurance = prices_df_without_insurance.drop_duplicates("shipment_id")
    return prices_df_without_insurance


def calculate_margin_insurance(prices_df: pd.DataFrame):
    prices_df_insurance = prices_df[prices_df["category"].isin(["insurance"])]
    prices_df_insurance.loc[:, "margin_insurance"] = prices_df_insurance.groupby("shipment_id")[
        "margin_price_in_eur"
    ].transform(sum)
    prices_df_insurance.loc[:, "initial_margin_insurance"] = prices_df_insurance.groupby("shipment_id")[
        "initial_margin_price_in_eur"
    ].transform(sum)
    prices_df_insurance = prices_df_insurance.drop_duplicates("shipment_id")
    return prices_df_insurance
